# FPS games notifications - notifications about occupied servers for Linux using https://api.bery.dev/
# Documentation: https://bery.dev/alien-arena-notifications
# Author: Bery <info@berycz.net>
# Version: 1.0


# first install python libraries PyGObject, websocket-client and rel:
# $ python3 -m pip install PyGObject websocket-client  # rel


# if more servers are occupied: I show only one (summary) notification
# I use only one notification instance and update it,
# cos I don't want to have 50 old notifications in the list in gnome


import json
import time
from pathlib import Path
import gi
gi.require_version('Notify', '0.7')  # this is on ubuntu, we will see about other distros
from gi.repository import Notify
import websocket
# import rel


class Notification:
    title = 'Alien Arena'
    icon = str(Path(__file__).parent / 'alienarena.ico')
    notification_instance = None
    last_notification_time = None
    min_notification_interval = 120
    occupied_servers = {}
    conky_settings = {
        "path": Path.home() / "conky" / "cache" / "alienarena"
    }

    def __call__(self, data):
        if not data:
            return

        if 'occupied' in data:
            self.occupied_servers = {f"{i['ip']}:{i['port']}": i for i in data['occupied']}
        elif 'occupation_changed' in data:
            key = f"{data['occupation_changed']['ip']}:{data['occupation_changed']['port']}"
            if data['occupation_changed'].get('humans'):
                self.occupied_servers[key] = data['occupation_changed']
            elif key in self.occupied_servers:
                del self.occupied_servers[key]

        self.gen_conky_template()

        if not self.occupied_servers:
            return

        message = self.get_message()

        if not message:
            return

        if self.notification_instance is None:
            Notify.init("AlienArena")
            self.notification_instance = Notify.Notification.new(self.title, message, self.icon)
        else:
            self.notification_instance.update(self.title, message, self.icon)

        now = time.time()
        if self.last_notification_time is None or self.last_notification_time < (now - self.min_notification_interval):
            self.notification_instance.show()
            self.last_notification_time = now

    def get_message(self):
        if not self.occupied_servers:
            return ''

        all_servers = []
        all_players = []

        for server in self.occupied_servers.values():
            all_servers.append(server['hostname'])
            all_players.extend([i['name'] for i in server['humans']])

        if (cnt_players := len(all_players)) <= 2:
            # 1 player: show '{name}'
            # 2 players: show '{name1} and {name2}'
            msg_players = u' and '.join(all_players)

        elif cnt_players == 3:
            # 3 players: show '{name1}, {name2} and {name3}'
            msg_players = u'%s, %s and %s' % tuple(all_players)

        else:
            # more players than 3: show '{name1}, {name2} and 2 others'
            msg_players = u'%s and %s others' % (u', '.join(all_players[:2]), len(all_players[2:]))

        if (cnt_servers := len(all_servers)) == 1:
            msg_servers = all_servers[0]
        else:
            msg_servers = f'{cnt_servers} servers'

        # I had " are currently playing", but it was too long for gnome notification
        return '{} {} playing on {}.'.format(
            msg_players,
            'is' if cnt_players == 1 else 'are',
            msg_servers
        )

    def gen_conky_template(self):
        """
        generates code for conky and saves it as a file
        """
        if not self.conky_settings:
            return

        # max number of servers
        show_max = self.conky_settings.get('max-servers', 8)

        # use color in hostnames and player names
        if self.conky_settings.get('use-colors'):
            use_colors = True
            if 're_colors' not in self.conky_settings:
                self.conky_settings['re_colors'] = re.compile('\^([0-9])([^^]*)')
            if 'name_colors' not in self.conky_settings:
                self.conky_settings['name_colors'] = [
                    '#ff0000',  # ^0
                    '#ff0000',  # ^1
                    '#000000',  # ^2
                    '#ffff00',  # ^3
                    '#0000ff',  # ^4
                    '#00ffff',  # ^5
                    '#ff00ff',  # ^6
                    '#ffffff',  # ^7
                    '#000000',  # ^8
                    '#ff0000',  # ^9
                ]
        else:
            use_colors = False

        empty_msg = '${font Technical CE:italic:size=16}${color8}No players online$color$font'

        result = []

        if self.occupied_servers:
            for server in self.occupied_servers.values():
                if use_colors:
                    hostname = server.get('hostname_full', server.get('hostname')) or 'unknown-server-name'
                else:
                    hostname = server.get('hostname') or 'unknown-server-name'
                code = ''
                address = f"{server['ip']}:{server['port']}"
                if '^' in hostname and use_colors:  # replace colors in hostname
                    hostname = self.conky_settings['re_colors'].sub((lambda m: '${color #%s}%s' % (self.conky_settings['name_colors'][int(m.group(1))], m.group(2))), hostname)
                    line = ['${font ae_AlMothnna:bold:size=16}%s' % hostname]
                else:
                    line = ['${font ae_AlMothnna:bold:size=16}${color9}%s' % hostname]

                line.append('${color8}(%s%s)$color$font' % (address, code))

                if server.get('humans'):
                    for player in server['humans']:
                        if use_colors:
                            name = player.get('name_full', player.get('name')) or 'unknown-name'
                        else:
                            name = player.get('name') or 'unknown-name'
                        if '^' in name and use_colors:  # replace colors in name
                            name = self.conky_settings['re_colors'].sub((lambda m: '${color #%s}%s' % (self.conky_settings['name_colors'][int(m.group(1))], m.group(2))), name)
                            line.append('    %s    ${color}(score: %s, ping: %s)' % (name, player.get('score') or 0, player.get('ping') or '?'))
                        else:
                            line.append('    ${color}%s    (score: %s, ping: %s)' % (name, player.get('score') or 0, player.get('ping') or '?'))

                result.append('\n'.join(line))

        else:
            result.append(empty_msg)

        result = u'\n\n'.join(result[:show_max])

        result = result.replace('#', '')

        self.conky_settings['path'].parent.mkdir(parents=True, exist_ok=True)
        self.conky_settings['path'].write_text(result, encoding='utf-8')


notify = Notification()


def on_message(ws, message):
    notify(json.loads(message))


def on_error(ws, error):
    print(error)


def on_close(ws, close_status_code, close_msg):
    print("### closed ###")


def on_open(ws):
    print("Opened connection")
    ws.send(json.dumps({"occupied": True}))


if __name__ == "__main__":

    # websocket.enableTrace(True)
    ws = websocket.WebSocketApp("wss://api.bery.dev/ws/fps/alienarena/",
                              on_open=on_open,
                              on_message=on_message,
                              on_error=on_error,
                              on_close=on_close)

    # ws.run_forever(dispatcher=rel)  # Set dispatcher to automatic reconnection
    ws.run_forever(reconnect=1)

    # rel.signal(2, rel.abort)  # Keyboard Interrupt
    # rel.dispatch()
